package com.hwrj.cloud.admin.dao;

import com.hwrj.cloud.admin.dto.param.*;
import com.hwrj.cloud.admin.model.PmsBrand;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
@Mapper
public interface PmsProductExcelDao {
    void saveData(@Param("list") List<PmsProductExcelParam> list);

    void saveBrandData(@Param("list") List<PmsBrand> list);

    List<BigDataListInfoVo> bigDataList(BigDataListParam bigDataListParam);

    BigDataListInfoVo bigDatainfo(@Param("id")Long id);

    List<SelectVo> bigDataType();

    Long checkOne(@Param("check")String check);

    List<SelectVo> plOfProSelect(@Param("compId")Long compId);

    void updateBrand(@Param("list")List<String> list,@Param("ids") List<Long> ids);

    void updateBigType(@Param("list")List<String> list,@Param("ids") List<Long> ids);

    void updateSimType(@Param("list")List<String> list,@Param("ids") List<Long> ids);

    void updateMemberOrigin(@Param("list")List<String> list,@Param("ids") List<Long> ids);

    void updateBrandComp(@Param("list")List<String> list,@Param("ids") List<Long> ids);

    void updateComp(@Param("list")List<String> list,@Param("ids") List<Long> ids );
}
